#include <stdlib.h>
#include <QVector>
#include <QString>
#include <QTextStream>

QTextStream cout(stdout, QIODevice::WriteOnly);
//start id=shift
/* shift() works with the QChars of the given QString */
QString shift(const QString& text, unsigned key) {
     const ushort P(127);   // The size of the ASCII char set.
     srandom(key);
     QString cryptstr;
     ushort ccode, shiftcode;
     int rnd;
     for (int i = 0; i < text.length(); ++i) {
         rnd = random();
         ccode = text[i].unicode();
         shiftcode = (ccode + rnd) % P ;
         cryptstr[i] = QChar(shiftcode) ;
     }
     return cryptstr;
 }
//end
//start id=unshift
  QString unshift(const QString& text, unsigned key) {
     const ushort P(127); 
     srandom(key);
     QString cryptstr;
     int temp;
     ushort ccode, uscode;
     int rnd;
     for (int i = 0; i < text.length(); ++i) {
         rnd = random();
         ccode = text[i].unicode();
         temp = (ccode - rnd) % P ;  // may be negative
         temp = (temp + P) % P;   // make sure it's positive
         uscode = temp;   // convert back to ushort
         cryptstr[i] = QChar(uscode) ;
     }
     return cryptstr;
 }
//end
//start id=myRand
int myRand(int min, int max) {
    return min + random() % (1 + max - min) ;
}
//end
//start id=randomPerm
void showVect(QVector<int> vec) {
    int vsize = vec.size();
    for (int i = 0; i < vsize; ++i)
        cout << vec[i] << "  ";
    cout << endl;   
}

 //Here is one way to obtain a random permutation of the numbers 0, ..., n :
 QVector<int> randomPerm(int n, unsigned key) {
     QVector<int> newperm(n) ; 
     newperm.fill(-1);
     srandom(key);
     int k;
     for (int i = 0; i < n; ++i) {
         while (1)  {
             k = myRand(0, n-1);
             if (newperm[k] != -1) continue;
             newperm[k] = i;
             break;
         }
     }
     return newperm;
 }
//end
//start id=permute
 QString permute(const QString& text, unsigned key) {
     int slen = text.length();
     QVector<int> perm = randomPerm(slen, key);
     QString scrstr;
     for (int i = 0; i < slen; ++i) 
         scrstr += text.at(perm[i]);
     return scrstr;
 }
 //end
 //start id=unpermute
 QString unpermute(const QString& text, unsigned key) {
     int slen = text.length();
     QVector<int> perm = randomPerm(slen, key);
     QString uscrstr;
     for (int i = 0; i < slen; ++i) 
         uscrstr[perm[i]] = text[i];
     return uscrstr;
 }
 //end
 //start id=testcode
 int main()  {
     QString str1 ("This is a sample string"), str2;
     cout << "Original string: " << str1 << endl;
     str2 = shift(str1, 1234);
     cout << "Shifted string: " << str2 << endl;
     cout << "Recovered string: " << unshift(str2, 1234) << endl;
     const int vsize(20);
     QVector<int> vec(vsize);
     vec = randomPerm(vsize, 1234);
     showVect(vec);
     str2 = permute(str1, 1234);
     cout << "Original string permuted: " << str2 << endl;
     cout << "Unpermuted string: " << unpermute(str2, 1234) << endl;
     QString str3 = shift(str2, 1234);
     cout << "Shifted, permuted string: " << str3 << endl;
     cout << "Recovered string: " << unpermute(unshift(str3, 1234), 1234)
             << endl;
}
 //end
 
